var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/index.ts
import { lookup } from "node:dns/promises";
import { SocksClient } from "socks";
var name = "http-socks";
var inject = ["http"];
function apply(ctx) {
  ctx.http.proxy(["socks", "socks4", "socks4a", "socks5", "socks5h"], (url) => {
    let shouldLookup = false;
    let type;
    const port = parseInt(url.port, 10) || 1080;
    const host = url.hostname;
    switch (url.protocol.slice(0, -1)) {
      case "socks4":
        shouldLookup = true;
      // eslint-disable-next-line no-fallthrough
      case "socks4a":
        type = 4;
        break;
      case "socks5":
        shouldLookup = true;
      // eslint-disable-next-line no-fallthrough
      case "socks":
      case "socks5h":
        type = 5;
        break;
      default:
        throw new Error("unreachable");
    }
    const proxy = { host, port, type };
    if (url.username) proxy.userId = decodeURIComponent(url.username);
    if (url.password) proxy.password = decodeURIComponent(url.password);
    return new ctx.http.undici.Agent({ connect: createConnect(proxy, shouldLookup) });
  });
  function resolvePort(protocol, port) {
    return port ? Number.parseInt(port) : protocol === "http:" ? 80 : 443;
  }
  __name(resolvePort, "resolvePort");
  function createConnect(proxy, shouldLookup, tlsOpts = {}) {
    const { timeout = 1e4 } = tlsOpts;
    const connect = ctx.http.undici.buildConnector(tlsOpts);
    return async (options, callback) => {
      let { protocol, hostname, port, httpSocket } = options;
      try {
        if (shouldLookup) {
          hostname = (await lookup(hostname)).address;
        }
        const event = await SocksClient.createConnection({
          command: "connect",
          proxy,
          timeout,
          destination: {
            host: hostname,
            port: resolvePort(protocol, port)
          },
          existing_socket: httpSocket
        });
        httpSocket = event.socket;
      } catch (error) {
        return callback(error, null);
      }
      if (httpSocket && protocol !== "https:") {
        return callback(null, httpSocket.setNoDelay());
      }
      return connect({ ...options, httpSocket }, callback);
    };
  }
  __name(createConnect, "createConnect");
}
__name(apply, "apply");
export {
  apply,
  inject,
  name
};
